@@ -1,4 +1,3 @@
-# coding=utf-8
 # Copyright 2022 Google LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,27 +15,34 @@
 # Lint as: python3
 """Implements a geometry class for points supported on a cartesian product."""
 import itertools
-from typing import Optional, Sequence
+from typing import Any, List, NoReturn, Optional, Sequence, Tuple
 
 import jax
 import jax.numpy as jnp
 import numpy as np
-from ott.geometry import costs
-from ott.geometry import geometry
-from ott.geometry import ops
+
+from ott.geometry import costs, geometry, pointcloud
+from ott.math import utils
+
+__all__ = ["Grid"]
 
 
 @jax.tree_util.register_pytree_node_class
 class Grid(geometry.Geometry):
-  r"""Class describing the geometry of points taken in a cartestian product.
+  r"""Class describing the geometry of points taken in a Cartesian product.
 
   This class implements a geometry in which probability measures are supported
   on a :math:`d`-dimensional cartesian grid, a cartesian product of :math:`d`
   lists of values, each list being itself of size :math:`n_i`.
 
   The transportation cost between points in the grid is assumed to be separable,
-  namely a sum of coordinate-wise cost functions, as in
-    :math:`cost(x,y) = \sum_{i=1}^d cost_i(x_i, y_i)` where :math:`cost_i`: R x R → R.
+  namely a sum of coordinate-wise cost functions, as in:
+
+  .. math::
+
+    cost(x,y) = \sum_{i=1}^d cost_i(x_i, y_i)
+
+  where :math:`cost_i`: R x R → R.
 
   In such a regime, and despite the fact that the total number :math:`n_{total}`
   of points in the grid is exponential :math:`d` (namely :math:`\prod_i n_i`),
@@ -46,6 +52,25 @@ class Grid(geometry.Geometry):
   precomputes :math:`d` :math:`n_i` x :math:`n_i` cost matrices (one per
   dimension) and implements these two operations by carrying out these
   convolutions one dimension at a time.
+
+  Args:
+    x : list of arrays of varying sizes, describing the locations of the grid.
+      Locations are provided as a list of arrays, that is :math:`d`
+      vectors of (possibly varying) size :math:`n_i`. The resulting grid
+      is the Cartesian product of these vectors.
+    grid_size: tuple of integers describing grid sizes, namely
+      :math:`(n_1,...,n_d)`. This will only be used if x is None.
+      In that case the grid will be assumed to lie in the hypercube
+      :math:`[0,1]^d`, with the :math:`d` dimensions, described as points
+      regularly sampled in [0,1].
+    cost_fns: a sequence of :math:`d` costs.CostFn's, each being a cost taking
+      two reals as inputs to output a real number.
+    num_a: total size of grid. This parameters will be computed from other
+      inputs and used in the flatten/unflatten functions.
+    grid_dimension: dimension of grid. This parameters will be computed from
+      other inputs and used in the flatten/unflatten functions.
+    kwargs: other optional parameters to be passed on to superclass
+      initializer, notably those related to epsilon regularization.
   """
 
   def __init__(
@@ -55,102 +80,89 @@ class Grid(geometry.Geometry):
       cost_fns: Optional[Sequence[costs.CostFn]] = None,
       num_a: Optional[int] = None,
       grid_dimension: Optional[int] = None,
-      **kwargs):
-    """Create instance of grid using either locations or sizes.
-
-    Args:
-      x : list of arrays of varying sizes, describing the locations of the grid.
-        Locations are provided as a list of jnp.ndarrays, that is :math:`d`
-        vectors of (possibly varying) size :math:`n_i`. The resulting grid
-        is the Cartesian product of these vectors.
-      grid_size: t-uple of integers describing grid sizes, namely
-        :math:`(n_1,...,n_d)`. This will only be used if x is None.
-        In that case the grid will be assumed to lie in the hypercube
-        :math:`[0,1]^d`, with the :math:`d` dimensions, described as points
-        regularly sampled in [0,1].
-      cost_fns: a sequence of :math:`d` costs.CostFn's, each being a cost taking
-        two reals as inputs to output a real number.
-      num_a: total size of grid. This parameters will be computed from other
-        inputs and used in the flatten/unflatten functions.
-      grid_dimension: dimension of grid. This parameters will be computed from
-        other inputs and used in the flatten/unflatten functions.
-      **kwargs: other optional parameters to be passed on to superclass
-        initializer, notably those related to epsilon regularization.
-
-    """
-    if (grid_size is not None and x is not None and num_a is not None and
-        grid_dimension is not None):
+      **kwargs: Any,
+  ):
+    if (
+        grid_size is not None and x is not None and num_a is not None and
+        grid_dimension is not None
+    ):
       self.grid_size = tuple(map(int, grid_size))
       self.x = x
       self.num_a = num_a
       self.grid_dimension = grid_dimension
     elif x is not None:
       self.x = x
-      self.grid_size = tuple([xs.shape[0] for xs in x])
+      self.grid_size = tuple(xs.shape[0] for xs in x)
       self.num_a = np.prod(np.array(self.grid_size))
       self.grid_dimension = len(self.x)
     elif grid_size is not None:
       self.grid_size = tuple(map(int, grid_size))
-      self.x = tuple([jnp.linspace(0, 1, n) for n in self.grid_size])
+      self.x = tuple(jnp.linspace(0, 1, n) for n in self.grid_size)
       self.num_a = np.prod(np.array(grid_size))
       self.grid_dimension = len(self.grid_size)
     else:
       raise ValueError('Input either grid_size t-uple or grid locations x.')
 
     if cost_fns is None:
-      cost_fns = [costs.Euclidean()]
+      cost_fns = [costs.SqEuclidean()]
     self.cost_fns = cost_fns
-    self.kwargs = {'num_a': self.num_a, 'grid_size': self.grid_size,
-                   'grid_dimension': self.grid_dimension}
+    self.kwargs = {
+        'num_a': self.num_a,
+        'grid_size': self.grid_size,
+        'grid_dimension': self.grid_dimension
+    }
 
     super().__init__(**kwargs)
 
   @property
-  def cost_matrices(self):
-    # computes cost matrices along each dimension of the grid
+  def cost_matrices(self) -> List[jnp.ndarray]:
+    """Cost matrices along each dimension of the grid."""
     cost_matrices = []
     for dimension, cost_fn in itertools.zip_longest(
-        range(self.grid_dimension), self.cost_fns,
-        fillvalue=self.cost_fns[-1]):
+        range(self.grid_dimension), self.cost_fns, fillvalue=self.cost_fns[-1]
+    ):
       x_values = self.x[dimension][:, jnp.newaxis]
-      cost_matrix = jax.vmap(lambda x1: jax.vmap(lambda y1: cost_fn(x1, y1))  # pylint: disable=cell-var-from-loop
-                             (x_values))(x_values)  # pylint: disable=cell-var-from-loop
-      cost_matrices.append(cost_matrix)
+      cost_matrices.append(
+          pointcloud.PointCloud(x_values, cost_fn=cost_fn).cost_matrix
+      )
     return cost_matrices
 
   @property
-  def kernel_matrices(self):
-    # computes kernel matrices from cost matrices grid
+  def kernel_matrices(self) -> List[jnp.ndarray]:
+    """Kernel matrices along each dimension of the grid."""
     kernel_matrices = []
     for cost_matrix in self.cost_matrices:
       kernel_matrices.append(jnp.exp(-cost_matrix / self.epsilon))
     return kernel_matrices
 
   @property
-  def median_cost_matrix(self):
-    raise NotImplementedError('Median cost not implemented for grids')
+  def median_cost_matrix(self) -> NoReturn:
+    """Not implemented."""
+    raise NotImplementedError('Median cost not implemented for grids.')
 
   @property
-  def shape(self):
+  def shape(self) -> Tuple[int, int]:
     return self.num_a, self.num_a
 
   @property
-  def is_symmetric(self):
+  def is_symmetric(self) -> bool:
     return True
 
   # Reimplemented functions to be used in regularized OT
-  def apply_lse_kernel(self,
-                       f: jnp.ndarray,
-                       g: jnp.ndarray,
-                       eps: float,
-                       vec: Optional[jnp.ndarray] = None,
-                       axis: int = 0):
-    """Applies grid kernel in log space. See notes in parent class for use case.
+  def apply_lse_kernel(
+      self,
+      f: jnp.ndarray,
+      g: jnp.ndarray,
+      eps: float,
+      vec: Optional[jnp.ndarray] = None,
+      axis: int = 0
+  ) -> jnp.ndarray:
+    """Apply grid kernel in log space. See notes in parent class for use case.
 
     Reshapes vector inputs below as grids, applies kernels onto each slice, and
     then expands the outputs as vectors.
 
-    More implementation details in https://arxiv.org/pdf/1708.01955.pdf
+    More implementation details in :cite:`schmitz:18`.
 
     Args:
       f: jnp.ndarray, a vector of potentials
@@ -179,31 +191,31 @@ class Grid(geometry.Geometry):
     return g.ravel(), vec.ravel()
 
   def _apply_lse_kernel_one_dimension(self, dimension, f, g, eps, vec=None):
-    """Helper function to permute axis & apply the kernel on a single slice."""
+    """Permute axis & apply the kernel on a single slice."""
     indices = np.arange(self.grid_dimension)
     indices[dimension], indices[0] = 0, dimension
     f, g = jnp.transpose(f, indices), jnp.transpose(g, indices)
-    centered_cost = (f[:, jnp.newaxis, ...] + g[jnp.newaxis, ...]
-                     - jnp.expand_dims(
-                         self.cost_matrices[dimension],
-                         axis=tuple(range(2, 1 + self.grid_dimension)))
-                     ) / eps
+    centered_cost = (
+        f[:, jnp.newaxis, ...] + g[jnp.newaxis, ...] - jnp.expand_dims(
+            self.cost_matrices[dimension],
+            axis=tuple(range(2, 1 + self.grid_dimension))
+        )
+    ) / eps
 
     if vec is not None:
       vec = jnp.transpose(vec, indices)
-      softmax_res, softmax_sgn = ops.logsumexp(
-          centered_cost, b=vec, axis=1, return_sign=True)
-      return eps * jnp.transpose(softmax_res, indices), jnp.transpose(
-          softmax_sgn, indices)
-    else:
-      softmax_res = ops.logsumexp(centered_cost, axis=1)
-      return eps * jnp.transpose(softmax_res, indices), None
+      softmax_res, softmax_sgn = utils.logsumexp(
+          centered_cost, b=vec, axis=1, return_sign=True
+      )
+      return eps * jnp.transpose(softmax_res,
+                                 indices), jnp.transpose(softmax_sgn, indices)
+    softmax_res = utils.logsumexp(centered_cost, axis=1)
+    return eps * jnp.transpose(softmax_res, indices), None
 
-  def _apply_cost_to_vec(self,
-                         vec: jnp.ndarray,
-                         axis: int = 0,
-                         fn=None) -> jnp.ndarray:
-    r"""Applies grid's cost matrix (without instantiating it) to a vector.
+  def _apply_cost_to_vec(
+      self, vec: jnp.ndarray, axis: int = 0, fn=None
+  ) -> jnp.ndarray:
+    r"""Apply grid's cost matrix (without instantiating it) to a vector.
 
     The `apply_cost` operation on grids rests on the following identity.
     If it were to be cast as a [num_a, num_a] matrix, the corresponding cost
@@ -239,21 +251,25 @@ class Grid(geometry.Geometry):
         cost = cost.T
       accum_vec += jnp.sum(
           jnp.tensordot(cost, vec, axes=([0], [dimension])),
-          axis=indices, keepdims=True).transpose(ind)
+          axis=indices,
+          keepdims=True
+      ).transpose(ind)
     return accum_vec.ravel()
 
-  def apply_kernel(self,
-                   scaling: jnp.ndarray,
-                   eps: Optional[float] = None,
-                   axis: Optional[int] = None):
-    """Applies grid kernel on scaling vector.
+  def apply_kernel(
+      self,
+      scaling: jnp.ndarray,
+      eps: Optional[float] = None,
+      axis: Optional[int] = None
+  ) -> jnp.ndarray:
+    """Apply grid kernel on scaling vector.
 
     See notes in parent class for use.
 
     Reshapes scaling vector as a grid, applies kernels onto each slice, and
     then ravels backs the output as a vector.
 
-    More implementation details in https://arxiv.org/pdf/1708.01955.pdf
+    More implementation details in :cite:`schmitz:18`,
 
     Args:
       scaling: jnp.ndarray, a vector of scaling (>0) values.
@@ -269,25 +285,56 @@ class Grid(geometry.Geometry):
       ind = indices.copy()
       ind.insert(dimension, 0)
       kernel = kernel if eps is None else kernel ** (self.epsilon / eps)
-      scaling = jnp.tensordot(kernel, scaling,
-                              axes=([0], [dimension])).transpose(ind)
+      scaling = jnp.tensordot(
+          kernel, scaling, axes=([0], [dimension])
+      ).transpose(ind)
     return scaling.ravel()
 
-  def transport_from_potentials(self, f: jnp.ndarray, g: jnp.ndarray, axis=0):
-    raise ValueError('Grid geometry cannot instantiate a transport matrix, use',
-                     ' apply_transport_from_potentials(...) if you wish to ',
-                     ' apply the transport matrix to a vector, or use a point '
-                     ' cloud geometry instead')
+  def transport_from_potentials(
+      self, f: jnp.ndarray, g: jnp.ndarray, axis: int = 0
+  ) -> NoReturn:
+    """Not implemented, use :meth:`apply_transport_from_potentials` instead."""
+    raise ValueError(
+        'Grid geometry cannot instantiate a transport matrix, use',
+        ' apply_transport_from_potentials(...) if you wish to ',
+        ' apply the transport matrix to a vector, or use a point '
+        ' cloud geometry instead'
+    )
 
-  def transport_from_scalings(self, f: jnp.ndarray, g: jnp.ndarray, axis=0):
-    raise ValueError('Grid geometry cannot instantiate a transport matrix, use',
-                     ' apply_transport_from_scalings(...) if you wish to ',
-                     ' apply the transport matrix to a vector, or use a point '
-                     ' cloud geometry instead')
+  def transport_from_scalings(
+      self, f: jnp.ndarray, g: jnp.ndarray, axis: int = 0
+  ) -> NoReturn:
+    """Not implemented, use :meth:`apply_transport_from_scalings` instead."""
+    raise ValueError(
+        'Grid geometry cannot instantiate a transport matrix, use ',
+        'apply_transport_from_scalings(...) if you wish to ',
+        'apply the transport matrix to a vector, or use a point '
+        'cloud geometry instead.'
+    )
+
+  def subset(
+      self, src_ixs: Optional[jnp.ndarray], tgt_ixs: Optional[jnp.ndarray]
+  ) -> NoReturn:
+    """Not implemented."""
+    raise NotImplementedError("Subsetting is not implemented for grids.")
+
+  def mask(
+      self,
+      src_mask: Optional[jnp.ndarray],
+      tgt_mask: Optional[jnp.ndarray],
+      mask_value: float = 0.,
+  ) -> NoReturn:
+    """Not implemented."""
+    raise NotImplementedError("Masking is not implemented for grids.")
 
   @classmethod
-  def prepare_divergences(cls, *args, static_b: bool = False, **kwargs):
-    """Instantiates the geometries used for a divergence computation."""
+  def prepare_divergences(
+      cls,
+      *args: Any,
+      static_b: bool = False,
+      **kwargs: Any
+  ) -> Tuple["Grid", ...]:
+    """Instantiate the geometries used for a divergence computation."""
     grid_size = kwargs.pop('grid_size', None)
     x = kwargs.pop('x', args)
 
@@ -295,10 +342,15 @@ class Grid(geometry.Geometry):
     size = 2 if static_b else 3
     return tuple(sep_grid for _ in range(size))
 
+  @property
+  def dtype(self) -> jnp.dtype:
+    return self.x[0].dtype
+
   def tree_flatten(self):
     return (self.x, self.cost_fns, self._epsilon), self.kwargs
 
   @classmethod
   def tree_unflatten(cls, aux_data, children):
     return cls(
-        x=children[0], cost_fns=children[1], epsilon=children[2], **aux_data)
+        x=children[0], cost_fns=children[1], epsilon=children[2], **aux_data
+    )
